from TestStack.White.UIItems import Label
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class LabelKeywords(LibraryComponent):
    @keyword
    def verify_label(self, locator, expected):
        """
        Verifies text of a label.

        ``locator`` is the locator of the label.

        ``expected`` is the expected text.
        """
        label = self.state._get_typed_item_by_locator(Label, locator)
        self.state._verify_value(expected, label.Text)

    @keyword
    def get_text_from_label(self, locator):
        """
        Gets text from a label.

        ``locator`` is the locator of the label.

        """
        label = self.state._get_typed_item_by_locator(Label, locator)
        return label.Text



from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White.UIItems.WindowItems import Window   # noqa: F401
from robot.api import logger    # noqa: F401


class WindowKeywords(LibraryComponent):
    @keyword
    def attach_window(self, window_title):
        """Attach WhiteLibrary to a window of already attached application.

        See `Attach Application By Id`, `Attach Application By Name` or
        `Launch application` for more details.

        ``window_title`` is the title of the window.
        """
        self.state.window = self.state.app.GetWindow(window_title)

    @keyword
    def select_modal_window(self, window_title):
        """Select modal window.

        ``window_title`` is the title of the window.
        """
        self.state.window = self.state.window.ModalWindow(window_title)



import clr
import os
from robot.api import logger    # noqa: F401
dll_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'bin', 'TestStack.White.dll')
clr.AddReference('System')
clr.AddReference(dll_path)
from System.Windows.Automation import AutomationElement, ControlType    # noqa: E402
from TestStack.White.UIItems.Finders import SearchCriteria    # noqa: E402
from WhiteLibrary.keywords import ApplicationKeywords, KeyboardKeywords, WindowKeywords, ScreenshotKeywords, WhiteConfigurationKeywords    # noqa: E402
from WhiteLibrary.keywords.items import (ButtonKeywords,
                            LabelKeywords,
                            ListKeywords,
                            MenuKeywords,
                            ProgressbarKeywords,
                            SliderKeywords,
                            TabKeywords,
                            TreeKeywords,
                            TextBoxKeywords,
                            UiItemKeywords)   # noqa: E402
from WhiteLibrary.keywords.robotlibcore import DynamicCore   # noqa: E402
from WhiteLibrary import version   # noqa: E402


STRATEGIES = dict(id={"method": "ByAutomationId"},
                  text={"method": "ByText"},
                  index={"method": "Indexed"},
                  help_text={"method": "ByNativeProperty", "property": "HelpTextProperty"},
                  class_name={"method": "ByClassName"},
                  control_type={"method": "ByControlType"})


class WhiteLibrary(DynamicCore):
    """WhiteLibrary is a Robot Framework library for automating Windows GUI.
    It is a wrapper for [https://github.com/TestStack/White | TestStack.White].

    = Applications and windows =
    To interact with UI items, the correct application and window must be attached to WhiteLibrary.

    When application is started with `Launch Application`, the keyword also attaches the application to WhiteLibrary.
    Attaching a running application is done with `Attach Application By Name` or `Attach Application By Id`.

    Once the application is attached, the window to interact with is attached with `Attach Window`.

    Examples:

    | # Launch application, no separate step for attaching application needed | |
    | `Launch Application` | C:/myApplication.exe |
    | `Attach Window`      | Main window |
    | | |
    | # Switch to an application that is already running | |
    | `Attach Application By Name` | calc1 |
    | `Attach Window`              | Calculator |

    = UI items =
    Keywords in WhiteLibrary use the same names for control types that are used in White.
    See [https://teststackwhite.readthedocs.io/en/latest/UIItems | White's documentation] for details about mapping
    UIA Control Types to classes in White.

    == Item locators ==
    Keywords that access UI items (e.g. `Click Button`) use a ``locator`` argument.
    The locator consists of a locator prefix that specifies the search criteria, and the locator value.

    The following locator prefixes are available:

    | = Prefix =        | = Description =                    |
    | id (or no prefix) | Search by AutomationID. If no prefix is given, the item is searched by AutomationID by default. |
    | text              | Search by exact item text or name. |
    | index             | Search by item index.              |
    | help_text         | Search by HelpTextProperty.        |
    | class_name        | Search by class name.              |
    | control_type      | Search by control type.            |

    Examples:

    | `Click Button` | myButton         | # clicks button by its AutomationID |
    | `Click Button` | id=myButton      | # clicks button by its AutomationID |
    | `Click Button` | text=Click here! | # clicks button by the button text  |
    | `Click Button` | index=2          | # clicks button whose index is 2    |
    """
    ROBOT_LIBRARY_VERSION = version.VERSION
    ROBOT_LIBRARY_SCOPE = "Global"
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.window = None
        self.screenshooter = None
        self.ROBOT_LIBRARY_LISTENER = self
        self.screenshot_type = 'desktop'
        self.screenshots_enabled = True
        self.libraries = [ApplicationKeywords(self),
                          ButtonKeywords(self),
                          KeyboardKeywords(self),
                          LabelKeywords(self),
                          ListKeywords(self),
                          MenuKeywords(self),
                          ProgressbarKeywords(self),
                          SliderKeywords(self),
                          TabKeywords(self),
                          WhiteConfigurationKeywords(self),
                          TextBoxKeywords(self),
                          TreeKeywords(self),
                          UiItemKeywords(self),
                          WindowKeywords(self),
                          ScreenshotKeywords(self)]
        DynamicCore.__init__(self, self.libraries)

    def _get_typed_item_by_locator(self, item_type, locator):
        search_criteria = self._get_search_criteria(locator)
        return self.window.Get[item_type](search_criteria)

    def _get_item_by_locator(self, locator):
        search_criteria = self._get_search_criteria(locator)
        return self.window.Get(search_criteria)

    def _get_multiple_items_by_locator(self, locator):
        search_criteria = self._get_search_criteria(locator)
        return self.window.GetMultiple(search_criteria)

    def _get_search_criteria(self, locator):
        if "=" not in locator:
            locator = "id=" + locator

        search_strategy, locator_value = locator.split("=", 1)
        if search_strategy == "index":
            locator_value = int(locator_value)

        try:
            search_method = STRATEGIES[search_strategy]["method"]
        except KeyError:
            raise ValueError("'{}' is not a valid locator prefix".format(search_strategy))

        if search_method == "ByNativeProperty":
            property_name = STRATEGIES[search_strategy]["property"]
            property_name = getattr(AutomationElement, property_name)
            search_params = (property_name, locator_value)
        else:
            if search_method == "ByControlType":
                locator_value = getattr(ControlType, locator_value)
            search_params = (locator_value,)

        method = getattr(SearchCriteria, search_method)
        return method(*search_params)

    def _end_keyword(self, name, attrs):
        if attrs['status'] == 'FAIL':
            if self.screenshot_type == 'desktop' and self.screenshots_enabled:
                self.screenshooter.take_desktop_screenshot()

    def _contains_string_value(self, expected, actual, case_sensitive=True):
        expected_value = expected if not case_sensitive else expected.upper()
        actual_value = actual if not case_sensitive else actual.upper()

        if expected_value not in actual_value:
            raise AssertionError("Expected value {} not found in {}".format(expected, actual))

    def _verify_string_value(self, expected, actual, case_sensitive=True):
        expected_value = expected if not case_sensitive else expected.upper()
        actual_value = actual if not case_sensitive else actual.upper()

        if expected_value != actual_value:
            raise AssertionError("Expected value {}, but found {}".format(expected, actual))

    def _verify_value(self, expected, actual):
        if expected != actual:
            raise AssertionError("Expected value {}, but found {}".format(expected, actual))



from TestStack.White.UIItems.MenuItems import Menu
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class MenuKeywords(LibraryComponent):
    @keyword
    def verify_menu(self, locator, expected):
        """
        Verifies the text of a menu item.

        ``locator`` is the locator of the menu item.

        ``expected`` is the expected text of the menu item.
        """
        menu = self.state._get_typed_item_by_locator(Menu, locator)
        self.state._verify_value(expected, menu.Name)

    @keyword
    def click_menu_button(self, locator):
        """
        Clicks a menu button.

        ``locator`` is the locator of the menu button.
        """
        menu_button = self.state._get_typed_item_by_locator(Menu, locator)
        menu_button.Click()



from .application import ApplicationKeywords   # noqa: F401
from .keyboard import KeyboardKeywords   # noqa: F401
from .window import WindowKeywords   # noqa: F401
from .screenshot import ScreenshotKeywords   # noqa: F401
from .configuration import WhiteConfigurationKeywords   # noqa: F401



from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White.InputDevices import Keyboard   # noqa: F401
from TestStack.White.WindowsAPI import KeyboardInput


SPECIAL_KEYS = ["SHIFT", "CONTROL", "ALT", "LEFT_ALT", "RIGHT_ALT", "RETURN",
                "RIGHT", "BACKSPACE", "LEFT", "ESCAPE", "TAB", "HOME", "END",
                "UP", "DOWN", "INSERT", "DELETE", "CAPS", "F1", "F2", "F3",
                "F4", "F5", "F6", "F7", "F8", "F9", "F11", "F12", "F13", "F14",
                "F15", "F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23",
                "F24", "PAGEUP", "PAGEDOWN", "PRINT", "PRINTSCREEN", "SPACE",
                "NUMLOCK", "SCROLL", "LWIN", "RWIN"]


class KeyboardKeywords(LibraryComponent):
    @keyword
    def press_special_key(self, key):
        """
        Press a special key (Ctrl, tab, alt for example).

        ``key`` must be one of the following:
        SHIFT, CONTROL, ALT, LEFT_ALT, RIGHT_ALT, RETURN ,RIGHT, BACKSPACE,
        LEFT, ESCAPE, TAB, HOME, END, UP, DOWN, INSERT, DELETE, CAPS, F1, F2,
        F3, F4, F5, F6, F7, F8, F9, F11, F12, F13, F14,F15, F16, F17, F18, F19,
        F20, F21, F22, F23,F24, PAGEUP, PAGEDOWN, PRINT, PRINTSCREEN, SPACE,
        NUMLOCK, SCROLL, LWIN, RWIN
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.PressSpecialKey(attribute)

    @keyword
    def hold_special_key(self, key):
        """
        Hold down a special key.

        ``key`` must be one of the accepted special keys, see documentation of
        `Press Special Key`
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.HoldKey(attribute)

    @keyword
    def leave_special_key(self, key):
        """
        Leave a special key that was previously held down.

        ``key`` must be one of the accepted special keys, see documentation of
        `Press Special Key`
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.LeaveKey(attribute)

    @keyword
    def press_keys(self, keys):
        """
        Press a key or keys.

        ``keys`` is the key or keys to press.

        Example:
        | Press Keys | a |
        | Press Keys | Text with special characters! |
        """
        self.state.window.Keyboard.Enter(keys)



from TestStack.White.UIItems import ProgressBar
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class ProgressbarKeywords(LibraryComponent):
    @keyword
    def verify_progressbar_value(self, locator, expected):
        """
        Verifies the value of a progress bar.

        ``locator`` is the locator of the progress bar.

        ``expected`` is the expected value of the progress bar.
        """
        progressbar = self.state._get_typed_item_by_locator(ProgressBar, locator)
        self.state._verify_value(float(expected), progressbar.Value)

    @keyword
    def get_progressbar_value(self, locator):
        """
        Gets the value of a progress bar.

        ``locator`` is the locator of the progress bar.

        """
        progressbar = self.state._get_typed_item_by_locator(ProgressBar, locator)
        return progressbar.Value



from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White import Application


class ApplicationKeywords(LibraryComponent):
    @keyword
    def launch_application(self, sut_path):
        self.state.app = Application.Launch(sut_path)

    @keyword
    def attach_application_by_name(self, sut_name):
        """ Attaches to a running application by name.

        ``sut_name`` is the application process name.
        """
        self.state.app = Application.Attach(sut_name)

    @keyword
    def attach_application_by_id(self, sut_id):
        """ Attaches to a running application by id.

        ``sut_id`` is the application process id.
        """
        self.state.app = Application.Attach(int(sut_id))

    @keyword
    def close_application(self):
        """
        Closes the application.
        """
        self.state.app.Close()
        self.state.app = None
        self.state.window = None



from TestStack.White.UIItems import TextBox
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class TextBoxKeywords(LibraryComponent):
    @keyword
    def input_text_to_textbox(self, locator, input_value):
        """
        Writes text to a textbox.

        ``locator`` is the locator of the text box.

        ``input_value`` is the text to write.
        """
        textBox = self.state._get_typed_item_by_locator(TextBox, locator)
        textBox.Text = input_value

    @keyword
    def verify_text_in_textbox(self, locator, expected):
        """
        Verifies text in a text box.

        ``locator`` is the locator of the text box.

        ``expected`` is the expected text of the text box.
        """
        textbox = self.state._get_typed_item_by_locator(TextBox, locator)
        self.state._verify_value(expected, textbox.Text)

    @keyword
    def get_text_from_textbox(self, locator):
        """
        Gets text from text box.

        ``locator`` is the locator of the text box.

        """
        textbox = self.state._get_typed_item_by_locator(TextBox, locator)
        return textbox.Text



from TestStack.White.UIItems import Slider
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class SliderKeywords(LibraryComponent):
    @keyword
    def set_slider_value(self, locator, value):
        """
        Sets a slider to the specified value.

        ``locator`` is the locator of the slider.

        ``value`` is the value to set.
        """
        slider = self.state._get_typed_item_by_locator(Slider, locator)
        slider.Value = float(value)

    @keyword
    def verify_slider_value(self, locator, expected):
        """
        Verifies a slider value.

        ``locator`` is the locator of the slider.

        ``expected`` is the expected value of the slider.
        """
        slider = self.state._get_typed_item_by_locator(Slider, locator)
        self.state._verify_value(float(expected), slider.Value)

    @keyword
    def get_slider_value(self, locator):
        """
        Gets a slider value.

        ``locator`` is the locator of the slider.

        """
        slider = self.state._get_typed_item_by_locator(Slider, locator)
        return slider.Value



from .buttons import ButtonKeywords   # noqa: F401
from .label import LabelKeywords   # noqa: F401
from .listcontrols import ListKeywords   # noqa: F401
from .menu import MenuKeywords   # noqa: F401
from .progressbar import ProgressbarKeywords   # noqa: F401
from .slider import SliderKeywords   # noqa: F401
from .tab import TabKeywords   # noqa: F401
from .textbox import TextBoxKeywords   # noqa: F401
from .tree import TreeKeywords   # noqa: F401
from .uiitem import UiItemKeywords   # noqa: F401



from TestStack.White.Configuration import CoreAppXmlConfiguration
from TestStack.White.UIItems import DateFormat
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from robot.api import logger


class WhiteConfigurationKeywords(LibraryComponent):

    @keyword
    def set_white_busy_timeout(self, value):
        """Sets busy timeout for White Teststack

        ``value`` is integer or timeout in milli seconds. Default is 5000.

        """

        CoreAppXmlConfiguration.Instance.BusyTimeout = int(value)
        logger.info("White Busy Timeout set to" + str(CoreAppXmlConfiguration.Instance.BusyTimeout))
        return CoreAppXmlConfiguration.Instance.BusyTimeout
    @keyword
    def get_white_busy_timeout(self):
        """Gets busy timeout for White Teststack

        """
        return CoreAppXmlConfiguration.Instance.BusyTimeout

    @keyword
    def set_white_find_window_timeout(self, value):
        """Sets find window timeout for White Teststack

        ``value`` is integer or timeout in milli seconds. Default is 30000.

        """

        CoreAppXmlConfiguration.Instance.FindWindowTimeout = int(value)
        logger.info("White FindWindowTimeout set to" + str(CoreAppXmlConfiguration.Instance.FindWindowTimeout))
        return CoreAppXmlConfiguration.Instance.FindWindowTimeout
    @keyword
    def get_white_find_window_timeout(self):
        """Gets white find window timeout for White Teststack

        """
        return CoreAppXmlConfiguration.Instance.FindWindowTimeout

    @keyword
    def set_white_double_click_interval(self, value):
        """Sets DoubleClickInterval for White Teststack

        ``value`` is integer. DoubleClickInterval adds delay in double click action between clicks. Value is milliseconds. Default value is 0.

        """

        CoreAppXmlConfiguration.Instance.DoubleClickInterval = int(value)
        logger.info("White MaxElementSearchDepth set to" + str(CoreAppXmlConfiguration.Instance.DoubleClickInterval))
        return CoreAppXmlConfiguration.Instance.DoubleClickInterval
    @keyword
    def get_white_double_click_interval(self):
        """Gets DoubleClickInterval for White Teststack

        """
        return CoreAppXmlConfiguration.Instance.DoubleClickInterval



from TestStack.White.UIItems import UIItem   # noqa: F401
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class UiItemKeywords(LibraryComponent):
    @keyword
    def click_item(self, locator):
        """Click an item.

        ``locator`` is the locator of the item.
        """
        item = self.state._get_item_by_locator(locator)
        item.Click()

    @keyword
    def right_click_item(self, locator):
        """Right click an item.

        ``locator`` is the locator of the item.
        """
        item = self.state._get_item_by_locator(locator)
        item.RightClick()

    @keyword
    def double_click_item(self, locator):
        """Double click an item.

        ``locator`` is the locator of the item.
        """
        item = self.state._get_item_by_locator(locator)
        item.DoubleClick()

    @keyword
    def get_items(self, locator):
        """Return a list of items that match the given locator.

        ``locator`` is the locator of the item.
        """
        return self.state._get_multiple_items_by_locator(locator)

    @keyword
    def get_item(self, locator):
        """Returns first item of the list of items that match the given locator.

        ``locator`` is the locator of the item.
        """
        return self.state._get_item_by_locator(locator)



from TestStack.White.UIItems import Button, CheckBox, RadioButton
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class ButtonKeywords(LibraryComponent):
    @keyword
    def click_button(self, locator):
        """ Clicks button.

        ``locator`` is the locator of the button.
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        button.Click()

    @keyword
    def button_text_should_be(self, locator, expected_text, case_sensitive=True):
        """Verifies exact text in a button.

        ``locator`` is the locator of the button.

        ``expected_text`` is the expected button text.

        ``case_sensitive`` compare text in case sensitive matter. Defaults to True
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        self.state._verify_string_value(expected_text, button.Text, case_sensitive)

    @keyword
    def button_text_should_contain(self, locator, expected_text, case_sensitive=True):
        """Verifies expected text is found in a button.

        ``locator`` is the locator of the button.

        ``expected_text`` is the expected button text.

        ``case_sensitive`` compare text in case sensitive matter. Defaults to True
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        self.state._contains_string_value(expected_text, button.Text, case_sensitive)

    @keyword
    def verify_button(self, locator, expected):
        """*DEPRECATED.* Use `Button Text Should Be` or `Button Text Should Contain` instead.

        Verifies text in a button.

        ``locator`` is the locator of the button.

        ``expected`` is the expected button text.
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        self.state._verify_value(expected, button.Text)

    @keyword
    def select_radio_button(self, locator):
        """
        Selects a radio button.

        ``locator`` is the locator of the radio button.
        """
        radiobutton = self.state._get_typed_item_by_locator(RadioButton, locator)
        radiobutton.Select()

    @keyword
    def verify_radio_button(self, locator, expected):
        """
        Verifies state of a radio button.

        ``locator`` is the locator of the radio button.

        ``expected`` is the expected state (True/False).
        """
        radiobutton = self.state._get_typed_item_by_locator(RadioButton, locator)
        self.state._verify_value(bool(expected), radiobutton.IsSelected)

    @keyword
    def get_radio_button_state(self, locator):
        """
        Gets the state of a radio button.

        ``locator`` is the locator of the radio button.

        """
        radiobutton = self.state._get_typed_item_by_locator(RadioButton, locator)
        return radiobutton.IsSelected

    @keyword
    def toggle_check_box(self, locator):
        """
        Toggles a check box.

        ``locator`` is the locator of the check box.
        """
        checkbox = self.state._get_typed_item_by_locator(CheckBox, locator)
        checkbox.Toggle()

    @keyword
    def verify_check_box(self, locator, expected):
        """
        Verifies state of a check box.

        ``locator`` is the locator of the check box.

        ``expected`` is the expected state (True/False).
        """
        checkbox = self.state._get_typed_item_by_locator(CheckBox, locator)
        self.state._verify_value(bool(expected), checkbox.IsSelected)

    @keyword
    def get_check_box_state(self, locator):
        """
        Gets the state of a check box.

        ``locator`` is the locator of the check box.

        """
        checkbox = self.state._get_typed_item_by_locator(CheckBox, locator)
        return checkbox.IsSelected



import os
from setuptools import setup, find_packages

version_data = {}
with open(os.path.join("src", "WhiteLibrary", "version.py")) as f:
    exec(f.read(), version_data)

requirements = []
with open("requirements.txt", "r") as f:
    requirements = list(filter(lambda s: s!="", f.read().split("\n")))


def __path(filename):
    return os.path.join(os.path.dirname(__file__),
                        filename)


build = 0

if os.path.exists(__path("build.info")):
    build = open(__path("build.info")).read().strip()

if version_data["STABLE"]:
    version = "{}".format(version_data["VERSION"])
else:
    version = "{}.{}.pre".format(version_data["VERSION"], build)

setup(name="robotframework-whitelibrary",
      version=version,
      description="Windows GUI testing library for Robot Framework",
      author="SALabs",
      author_email="to.be.added@noexist89a887.org",
      url="https://github.com/Omenia/robotframework-whitelibrary",
      install_requires=requirements,
      packages=find_packages("src"),
      package_dir={"WhiteLibrary": "src/WhiteLibrary"},
      package_data={"WhiteLibrary": ["bin/*.dll"]},
      classifiers=["Programming Language :: Python :: 2",
                   "Programming Language :: Python :: 2.7",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6"],
      )



from TestStack.White.UIItems.ListBoxItems import ComboBox, ListBox
from TestStack.White.UIItems import ListView
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from robot.api import logger

class ListKeywords(LibraryComponent):
    @keyword
    def select_listbox_value(self, locator, value):
        """Selects a value from a listbox.

        ``locator`` is the locator of the listbox.

        ``value`` is the value to be selected.
        """
        listbox = self.state._get_typed_item_by_locator(ListBox, locator)
        listbox.Select(value)

    @keyword
    def select_listbox_index(self, locator, item_index):
        """Selects an item by its index from a listbox.

        ``locator`` is the locator of the listbox.

        ``item_index`` is the index of the item to select.
        """
        listbox = self.state._get_typed_item_by_locator(ListBox, locator)
        listbox.Select(int(item_index))

    @keyword
    def listbox_selection_should_be(self, locator, expected):
        """
        Checks the listbox selection.

        Fails if the selection was not as expected.

        ``locator`` is the locator of the listbox.

        ``expected`` is the expected selection value.
        """
        listbox = self.state._get_typed_item_by_locator(ListBox, locator)
        if listbox.SelectedItemText != expected:
            raise AssertionError("Expected listbox selection to be " +   # noqa: W504
                                 expected + ", was " + listbox.SelectedItemText)

    @keyword
    def select_combobox_value(self, locator, value):
        """
        Selects a value from a combobox.

        ``locator`` is the locator of the combobox.

        ``value`` is the value to be selected.
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        combobox.Select(value)

    @keyword
    def select_combobox_index(self, locator, item_index):
        """
        Selects a value from combobox by using its index.

        ``locator`` is the locator of the combobox.

        ``item_index`` is the index to be selected.
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        combobox.Select(int(item_index))

    @keyword
    def verify_combobox_item(self, locator, expected):
        """
        *DEPRECATED* Please use Verify Combobox Selection instead
        Verifies the selected value of a combobox.

        ``locator`` is the locator of the combobox.

        ``expected`` is the expected selection value.
        """
        self.verify_combobox_selection(locator, expected)

    @keyword
    def verify_combobox_selection(self, locator, expected):
        """
        Verifies the selected value of a combobox.

        ``locator`` is the locator of the combobox.

        ``expected`` is the expected selection value.
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        self.state._verify_value(expected, combobox.EditableText)

    @keyword
    def right_click_listview_cell(self, locator, column_name, row_index):
        """
        Selects a value from a listview.

        ``locator`` is the locator of the listview.

        ``column_name`` is the name of the column.

        ``row_index`` is the index of the row.
        """
        listview = self.state._get_typed_item_by_locator(ListView, locator)
        cell = listview.Cell(column_name, int(row_index))
        cell.RightClick()

    @keyword
    def select_listview_row_by_index(self, locator, row_index):
        """Selects a row from a listview.

        ``locator`` is the locator of the listview.

        ``row_index`` is the index to select.
        """
        listview = self.state._get_typed_item_by_locator(ListView, locator)
        listview.Select(int(row_index))




from TestStack.White.UIItems.TreeItems import Tree
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class TreeKeywords(LibraryComponent):
    @keyword
    def select_tree_node(self, locator, *node_path):
        """
        Selects a tree node.

        ``locator`` is the locator of the tree.

        ``node_path`` is the path the to node to select.

        Example tree (tree locator is tree_id):
        | root
        | |
        | |---parent_node
        | |   |
        | |   |---child_node
        | |   |
        | |   |---sibling_node
        | |
        | |---other parent

        Example usage to select ``sibling node``:
        | Select Tree Node | tree_id | root | parent_node | sibling_node |
        """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).Select()

    @keyword
    def expand_tree_node(self, locator, *node_path):
        """
        Expands a tree node.

        ``locator`` is the locator of the tree.

        ``node_path`` is the path the to node to expand.

        See examples of the node path in `Select Tree Node` documentation.
        """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).Expand()

    @keyword
    def double_click_tree_node(self, locator, *node_path):
        """
        Double-clicks a tree node.

        ``locator`` is the locator of the tree.

        ``node_path`` is the path the to node to double-click.

        See examples of the node path in `Select Tree Node` documentation.
        """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).DoubleClick()

    @keyword
    def right_click_tree_node(self, locator, *node_path):
        """
        Right-clicks a tree node.

        ``locator`` is the locator of the tree.

        ``node_path`` is the path the to node to right-click.

        See examples of the node path in `Select Tree Node` documentation.
        """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).RightClick()
